@@ -62,20 +62,27 @@ _item_display_name_changed_cb (AnerleyItem *item,
 {
   AnerleyTilePrivate *priv = GET_PRIVATE (userdata);
 
-  if (anerley_item_get_first_name (priv->item))
+  /* If first and last present then set as first and secondary, otherwise if
+   * just first then put that in primary or if just last then put that in
+   * primary. Last will always be set to something.
+   */
+  if (anerley_item_get_first_name (priv->item) &&
+      anerley_item_get_last_name (priv->item))
   {
     nbtk_label_set_text (NBTK_LABEL (priv->primary_label),
                          anerley_item_get_first_name (priv->item));
-  } else {
-    nbtk_label_set_text (NBTK_LABEL (priv->primary_label),
-                         "");
-  }
-
-  if (anerley_item_get_last_name (priv->item))
-  {
     nbtk_label_set_text (NBTK_LABEL (priv->secondary_label),
                          anerley_item_get_last_name (priv->item));
   } else {
+    if (anerley_item_get_first_name (priv->item))
+    {
+      nbtk_label_set_text (NBTK_LABEL (priv->primary_label),
+                           anerley_item_get_first_name (priv->item));
+    } else {
+      nbtk_label_set_text (NBTK_LABEL (priv->primary_label),
+                           anerley_item_get_last_name (priv->item));
+    }
+
     nbtk_label_set_text (NBTK_LABEL (priv->secondary_label),
                          "");
   }
@@ -93,8 +100,7 @@ _get_default_avatar_texture (void)
 
   cache = nbtk_texture_cache_get_default ();
   tmp_tex = nbtk_texture_cache_get_texture (cache,
-                                            DEFAULT_AVATAR_IMAGE,
-                                            FALSE);
+                                            DEFAULT_AVATAR_IMAGE);
   handle = clutter_texture_get_cogl_texture (tmp_tex);
   cogl_handle_ref (handle);
   g_object_unref (tmp_tex);
@@ -144,8 +150,7 @@ anerley_tile_update_presence_icon (AnerleyTile *tile,
   cache = nbtk_texture_cache_get_default ();
 
   priv->presence_icon = (ClutterActor *)nbtk_texture_cache_get_texture (cache,
-                                                                        path,
-                                                                        FALSE);
+                                                                        path);
   clutter_actor_set_size (priv->presence_icon, 16, 16);
   nbtk_table_add_actor_with_properties (NBTK_TABLE (tile),
                                         (ClutterActor *)priv->presence_icon,
@@ -366,6 +371,31 @@ anerley_tile_allocate (ClutterActor           *actor,
   actor_class->allocate (actor, box, flags);
 }
 
+static void
+anerley_tile_get_preferred_width (ClutterActor *actor,
+                                  gfloat        for_height,
+                                  gfloat       *min_width,
+                                  gfloat       *pref_width)
+{
+  if (min_width)
+    *min_width = 180;
+  if (pref_width)
+    *pref_width = 180;
+}
+
+static void
+anerley_tile_get_preferred_height (ClutterActor *actor,
+                                   gfloat        for_width,
+                                   gfloat       *min_height,
+                                   gfloat       *pref_height)
+{
+  if (min_height)
+    *min_height = 90;
+  if (pref_height)
+    *pref_height = 90;
+}
+
+
 static void
 anerley_tile_class_init (AnerleyTileClass *klass)
 {
@@ -382,6 +412,8 @@ anerley_tile_class_init (AnerleyTileClass *klass)
 
   actor_class->map = anerley_tile_map;
   actor_class->allocate = anerley_tile_allocate;
+  actor_class->get_preferred_width = anerley_tile_get_preferred_width;
+  actor_class->get_preferred_height = anerley_tile_get_preferred_height;
 
   pspec = g_param_spec_object ("item",
                                "Item",
@@ -399,7 +431,7 @@ _button_press_event_cb (ClutterActor *actor,
   AnerleyTilePrivate *priv = GET_PRIVATE (actor);
   ClutterActor *parent;
 
-  if (((ClutterButtonEvent *)event)->click_count > 1)
+  if (((ClutterButtonEvent *)event)->click_count == 2)
   {
     parent = clutter_actor_get_parent (actor);
 
@@ -443,94 +475,78 @@ anerley_tile_init (AnerleyTile *self)
   priv->avatar_bin = nbtk_bin_new ();
   nbtk_widget_set_style_class_name (priv->avatar_bin, "AnerleyTileAvatar");
 
-  priv->avatar = g_object_new (PENGE_TYPE_MAGIC_TEXTURE,
-                               NULL);
-  clutter_actor_set_size ((ClutterActor *)priv->avatar_bin, 64, 64);
+  priv->avatar = g_object_new (PENGE_TYPE_MAGIC_TEXTURE, NULL);
   /* TODO: Prefill with unknown icon */
   nbtk_bin_set_child (NBTK_BIN (priv->avatar_bin), priv->avatar);
 
-  nbtk_table_add_actor_with_properties (NBTK_TABLE (self),
-                                        (ClutterActor *)priv->avatar_bin,
-                                        0,
-                                        0,
-                                        "x-fill",
-                                        FALSE,
-                                        "y-fill",
-                                        FALSE,
-                                        "x-expand",
-                                        FALSE,
-                                        "y-expand",
-                                        FALSE,
-                                        "row-span",
-                                        3,
-                                        NULL);
+  /* add avatar */
+  nbtk_table_add_actor (NBTK_TABLE (self), (ClutterActor *)priv->avatar_bin, 0,
+                        0);
+  nbtk_table_child_set_x_fill (NBTK_TABLE (self),
+                               (ClutterActor *)priv->avatar_bin, FALSE);
+  nbtk_table_child_set_y_fill (NBTK_TABLE (self),
+                               (ClutterActor *)priv->avatar_bin, FALSE);
+  nbtk_table_child_set_x_expand (NBTK_TABLE (self),
+                                 (ClutterActor *)priv->avatar_bin, FALSE);
+  nbtk_table_child_set_y_expand (NBTK_TABLE (self),
+                                 (ClutterActor *)priv->avatar_bin, FALSE);
+  nbtk_table_child_set_row_span (NBTK_TABLE (self),
+                                 (ClutterActor *)priv->avatar_bin, 3);
 
   priv->primary_label = nbtk_label_new ("");
   nbtk_widget_set_style_class_name (priv->primary_label,
                                     "AnerleyTilePrimaryLabel");
-  nbtk_table_add_actor_with_properties (NBTK_TABLE (self),
-                                        (ClutterActor *)priv->primary_label,
-                                        0,
-                                        1,
-                                        "x-fill",
-                                        TRUE,
-                                        "y-fill",
-                                        FALSE,
-                                        "y-expand",
-                                        TRUE,
-                                        "x-expand",
-                                        FALSE,
-                                        "x-align",
-                                        0.0,
-                                        "y-align",
-                                        1.0,
-                                        "col-span",
-                                        2,
-                                        NULL);
+
+  nbtk_table_add_actor (NBTK_TABLE (self), (ClutterActor *)priv->primary_label,
+                        0, 1);
+  nbtk_table_child_set_y_fill (NBTK_TABLE (self),
+                               (ClutterActor *)priv->primary_label, FALSE);
+  nbtk_table_child_set_x_expand (NBTK_TABLE (self),
+                                 (ClutterActor *)priv->primary_label, FALSE);
+  nbtk_table_child_set_col_span (NBTK_TABLE (self),
+                                 (ClutterActor *)priv->primary_label, 2);
+  nbtk_table_child_set_y_align (NBTK_TABLE (self),
+                                (ClutterActor *)priv->primary_label, 1.0);
+  nbtk_table_child_set_x_align (NBTK_TABLE (self),
+                                (ClutterActor *)priv->primary_label, 0.0);
 
   priv->secondary_label = nbtk_label_new ("");
   nbtk_widget_set_style_class_name (priv->secondary_label,
                                     "AnerleyTileSecondaryLabel");
-  nbtk_table_add_actor_with_properties (NBTK_TABLE (self),
-                                        (ClutterActor *)priv->secondary_label,
-                                        1,
-                                        1,
-                                        "x-fill",
-                                        TRUE,
-                                        "y-fill",
-                                        FALSE,
-                                        "y-expand",
-                                        FALSE,
-                                        "x-expand",
-                                        FALSE,
-                                        "x-align",
-                                        0.0,
-                                        "y-align",
-                                        0.0,
-                                        "col-span",
-                                        2,
-                                        NULL);
+
+  nbtk_table_add_actor (NBTK_TABLE (self),
+                        (ClutterActor *)priv->secondary_label, 1, 1);
+  nbtk_table_child_set_y_fill (NBTK_TABLE (self),
+                               (ClutterActor *)priv->secondary_label,
+                               FALSE);
+  nbtk_table_child_set_y_expand (NBTK_TABLE (self),
+                                 (ClutterActor *)priv->secondary_label,
+                                 FALSE);
+  nbtk_table_child_set_x_expand (NBTK_TABLE (self),
+                                 (ClutterActor *)priv->secondary_label,
+                                   FALSE);
+  nbtk_table_child_set_col_span (NBTK_TABLE (self),
+                                 (ClutterActor *)priv->secondary_label, 2);
+  nbtk_table_child_set_y_align (NBTK_TABLE (self),
+                                (ClutterActor *)priv->secondary_label, 0.0);
+  nbtk_table_child_set_x_align (NBTK_TABLE (self),
+                                (ClutterActor *)priv->secondary_label, 0.0);
 
   priv->presence_label = nbtk_label_new ("");
   nbtk_widget_set_style_class_name (priv->presence_label,
                                     "AnerleyTilePresenceLabel");
-  nbtk_table_add_actor_with_properties (NBTK_TABLE (self),
-                                        (ClutterActor *)priv->presence_label,
-                                        2,
-                                        2,
-                                        "x-fill",
-                                        FALSE,
-                                        "y-fill",
-                                        FALSE,
-                                        "y-expand",
-                                        TRUE,
-                                        "x-expand",
-                                        TRUE,
-                                        "x-align",
-                                        0.0,
-                                        "y-align",
-                                        0.0,
-                                        NULL);
+
+  nbtk_table_add_actor (NBTK_TABLE (self),
+                        (ClutterActor *)priv->presence_label, 2, 2);
+  nbtk_table_child_set_y_fill (NBTK_TABLE (self),
+                               (ClutterActor *)priv->presence_label, FALSE);
+  nbtk_table_child_set_x_fill (NBTK_TABLE (self),
+                               (ClutterActor *)priv->presence_label, FALSE);
+  nbtk_table_child_set_y_align (NBTK_TABLE (self),
+                                (ClutterActor *)priv->presence_label, 0.0);
+  nbtk_table_child_set_x_align (NBTK_TABLE (self),
+                                (ClutterActor *)priv->presence_label, 0.0);
+
   nbtk_table_set_col_spacing ((NbtkTable *)self,
                               4);
   nbtk_table_set_row_spacing ((NbtkTable *)self,
@@ -551,11 +567,8 @@ anerley_tile_init (AnerleyTile *self)
 
   clutter_actor_set_reactive ((ClutterActor *)self, TRUE);
 
-  /* Need to set a fixed size */
-  clutter_actor_set_size ((ClutterActor *)self, 180, 90);
-
-  /* Need to hide it to take advantage of optimisations in grid */
-  clutter_actor_hide ((ClutterActor *)self);
+  /* Need stay hidden it to take advantage of optimisations in grid */
+  g_object_set ((ClutterActor *)self, "show-on-set-parent", FALSE, NULL);
 }
 
 NbtkWidget *
